/*
 *
 *
 *    Resea (x86_64/memory.c)
 *    Mafice <mafice@mflabo.net>
 *
 *
 */

#include "kernel.h"



/*
*  Function: isActiveMemoryAddress
*
*    存在する物理アドレス(メインメモリの容量内)かチェック
*
*  Parameters:
*
*    addr -  物理アドレス
*
*  Returns:
*
*    なし
*
*/

bool isActiveMemoryAddress (address_t addr){

 volatile u64  *p;

  setPagingTable((void *) PAGING_TABLE_ADDRESS, addr, BASE_ADDRESS+addr, PG_WRITABLE, PG_KERNEL);

  p  = (void *) addr;
  *p = 0x55aa55aa55aa55aa;

  return ((*p == 0)? false:true);
}




/*
*  Function: kmalloc
*
*    メモリを割り当てる
*
*  Parameters:
*
*    size - 取得するメモリのサイズ
*
*  Returns:
*
*    成功した場合はそのメモリのアドレス、失敗した場合はNULLを返します。
*
*/

void *kmalloc (size_t size){

 bool     flag;
 size_t   blockSize;
 index_t  i, j;


  flag = false;
  blockSize = (size/MEMORY_BLOCK_SIZE)+1;

  for(i=0;; i++){
    if(Memory.block[i].status == MEMORY_UNUSED){
      for(j=0; (Memory.block[i].status == MEMORY_UNUSED) && (j <= blockSize); i++, j++){
         if(j == blockSize){
           flag = true;
           goto ret;
          }
       }
    }
  }

 ret:

  if(!flag)
    return NULL;

  for(j = i; j < (blockSize+i); j++)
    Memory.block[j].status = MEMORY_USING;

  return (void *) (i*MEMORY_BLOCK_SIZE)+MEMORY_USERAREA_ADDRESS;
}




/*
*  Function: kfree
*
*    メモリを開放する
*
*  Parameters:
*
*    p   - メモリへのポインタ
*
*  Returns:
*
*    成功した場合はそのメモリのアドレス、失敗した場合はNULLを返します。
*
*/

void kfree (void *p){

 index_t i = ((address_t) p-MEMORY_USERAREA_ADDRESS)/MEMORY_BLOCK_SIZE;

  for(; Memory.block[i].status == MEMORY_USING; i++)
    Memory.block[i].status = MEMORY_UNUSED;

  return;
}

